@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: depcache.cc,v 1.25 2001/05/27 05:36:04 jgg Exp $
 /* ######################################################################
 
    Dependency Cache - Caches Dependency information.
@@ -10,7 +9,6 @@
 // Include Files							/*{{{*/
 #include <config.h>
 
-#include <apt-pkg/algorithms.h>
 #include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/cachefile.h>
 #include <apt-pkg/cacheset.h>
@@ -44,23 +42,6 @@
 
 using std::string;
 
-// helper for kernel autoremoval				  	/*{{{*/
-
-/** \brief Returns \b true for packages matching a regular
- *  expression in APT::NeverAutoRemove.
- */
-class DefaultRootSetFunc2 : public pkgDepCache::DefaultRootSetFunc
-{
-   std::unique_ptr<APT::CacheFilter::Matcher> Kernels;
-
-   public:
-   DefaultRootSetFunc2(pkgCache *cache) : Kernels(APT::KernelAutoRemoveHelper::GetProtectedKernelsFilter(cache)){};
-   virtual ~DefaultRootSetFunc2(){};
-
-   bool InRootSet(const pkgCache::PkgIterator &pkg) APT_OVERRIDE { return pkg.end() == false && ((*Kernels)(pkg) || DefaultRootSetFunc::InRootSet(pkg)); };
-};
-
-									/*}}}*/
 // helper for Install-Recommends-Sections and Never-MarkAuto-Sections	/*{{{*/
 static bool 
 ConfigValueInSubTree(const char* SubTree, const char *needle)
@@ -432,7 +413,7 @@ bool pkgDepCache::CheckDep(DepIterator const &Dep,int const Type,PkgIterator &Re
 									/*}}}*/
 // DepCache::AddSizes - Add the packages sizes to the counters		/*{{{*/
 // ---------------------------------------------------------------------
-/* Call with Inverse = true to preform the inverse opration */
+/* Call with Inverse = true to perform the inverse operation */
 void pkgDepCache::AddSizes(const PkgIterator &Pkg, bool const Inverse)
 {
    StateCache &P = PkgState[Pkg->ID];
@@ -498,10 +479,10 @@ void pkgDepCache::AddSizes(const PkgIterator &Pkg, bool const Inverse)
 									/*}}}*/
 // DepCache::AddStates - Add the package to the state counter		/*{{{*/
 // ---------------------------------------------------------------------
-/* This routine is tricky to use, you must make sure that it is never 
+/* This routine is tricky to use, you must make sure that it is never
    called twice for the same package. This means the Remove/Add section
-   should be as short as possible and not encompass any code that will 
-   calld Remove/Add itself. Remember, dependencies can be circular so
+   should be as short as possible and not encompass any code that will
+   call Remove/Add itself. Remember, dependencies can be circular so
    while processing a dep for Pkg it is possible that Add/Remove
    will be called on Pkg */
 void pkgDepCache::AddStates(const PkgIterator &Pkg, bool const Invert)
@@ -618,7 +599,7 @@ unsigned char pkgDepCache::VersionState(DepIterator D, unsigned char const Check
 // DepCache::DependencyState - Compute the 3 results for a dep		/*{{{*/
 // ---------------------------------------------------------------------
 /* This is the main dependency computation bit. It computes the 3 main
-   results for a dependencys, Now, Install and Candidate. Callers must
+   results for a dependency: Now, Install and Candidate. Callers must
    invert the result if dealing with conflicts. */
 unsigned char pkgDepCache::DependencyState(DepIterator const &D)
 {
@@ -1829,14 +1810,14 @@ APT_PURE signed short pkgDepCache::Policy::GetPriority(pkgCache::PkgFileIterator
 									/*}}}*/
 pkgDepCache::InRootSetFunc *pkgDepCache::GetRootSetFunc()		/*{{{*/
 {
-   DefaultRootSetFunc *f = new DefaultRootSetFunc2(&GetCache());
-   if (f->wasConstructedSuccessfully())
-      return f;
-   else
-   {
+  DefaultRootSetFunc *f = new DefaultRootSetFunc;
+  if(f->wasConstructedSuccessfully())
+    return f;
+  else
+    {
       delete f;
       return NULL;
-   }
+    }
 }
 									/*}}}*/
 bool pkgDepCache::MarkFollowsRecommends()
@@ -1892,28 +1873,29 @@ bool pkgDepCache::MarkRequired(InRootSetFunc &userFunc)
       if (PkgState[P->ID].Marked || IsPkgInBoringState(P, PkgState))
 	 continue;
 
+      const char *reason = nullptr;
+
       if ((PkgState[P->ID].Flags & Flag::Auto) == 0)
-	 ;
-      else if ((P->Flags & Flag::Essential) || (P->Flags & Flag::Important))
-	 ;
-      // be nice even then a required package violates the policy (#583517)
-      // and do the full mark process also for required packages
+	 reason = "Manual-Installed";
+      else if (P->Flags & Flag::Essential)
+	 reason = "Essential";
+      else if (P->Flags & Flag::Important)
+	 reason = "Important";
       else if (P->CurrentVer != 0 && P.CurrentVer()->Priority == pkgCache::State::Required)
-	 ;
+	 reason = "Required";
       else if (userFunc.InRootSet(P))
-	 ;
-      // packages which can't be changed (like holds) can't be garbage
+	 reason = "Blacklisted [APT::NeverAutoRemove]";
       else if (IsModeChangeOk(ModeGarbage, P, 0, false) == false)
-	 ;
+	 reason = "Hold";
       else
 	 continue;
 
       if (PkgState[P->ID].Install())
 	 MarkPackage(P, PkgState[P->ID].InstVerIter(*this),
-	       follow_recommends, follow_suggests);
+	       follow_recommends, follow_suggests, reason);
       else
 	 MarkPackage(P, P.CurrentVer(),
-	       follow_recommends, follow_suggests);
+	       follow_recommends, follow_suggests, reason);
    }
 
    return true;
@@ -1923,7 +1905,8 @@ bool pkgDepCache::MarkRequired(InRootSetFunc &userFunc)
 void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
 			      const pkgCache::VerIterator &Ver,
 			      bool const &follow_recommends,
-			      bool const &follow_suggests)
+			      bool const &follow_suggests,
+			      const char *reason)
 {
    {
       pkgDepCache::StateCache &state = PkgState[Pkg->ID];
@@ -1938,7 +1921,8 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
 
    bool const debug_autoremove = _config->FindB("Debug::pkgAutoRemove", false);
    if(debug_autoremove)
-      std::clog << "Marking: " << Pkg.FullName() << " " << Ver.VerStr() << std::endl;
+      std::clog << "Marking: " << Pkg.FullName() << " " << Ver.VerStr()
+		<< " (" << reason << ")" << std::endl;
 
    for (auto D = Ver.DependsList(); D.end() == false; ++D)
    {
@@ -1995,7 +1979,7 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
 	       std::clog << "Following dep: " << APT::PrettyDep(this, D)
 		  << ", provided by " << PP.FullName() << " " << PV.VerStr()
 		  << " (" << providers.size() << "/" << prvsize << ")"<< std::endl;
-	    MarkPackage(PP, PV, follow_recommends, follow_suggests);
+	    MarkPackage(PP, PV, follow_recommends, follow_suggests, "Provider");
 	 }
       }
 
@@ -2010,7 +1994,7 @@ void pkgDepCache::MarkPackage(const pkgCache::PkgIterator &Pkg,
 
       if (debug_autoremove)
 	 std::clog << "Following dep: " << APT::PrettyDep(this, D) << std::endl;
-      MarkPackage(T, TV, follow_recommends, follow_suggests);
+      MarkPackage(T, TV, follow_recommends, follow_suggests, "Dependency");
    }
 }
 									/*}}}*/
